class Controls
{
  int x, y;
  int size;
  boolean over;
  boolean press;
  boolean locked = false;
}


class CheckBox
{
  int x, y;
  int size;
  boolean over;
  boolean press;
  boolean checked = false;
  boolean allowMultiple = false;
  CheckBox[] others;

  CheckBox(int ix, int iy, int is)
  {
    x = ix;
    y = iy;
    size = is;
    others = null;
  }

  CheckBox(int ix, int iy, int is, CheckBox[] o)
  {
    x = ix;
    y = iy;
    size = is;
    others = o;
  }


  void over()
  {
    if (overRect(x, y, size, size)) {
      over = true;
    } 
    else {
      over = false;
    }
  }

  void press()
  {
    if (over && mousePressed ) {
      if (!press)
      {  
        press = true;
        if (others != null)
        {
          if (!checked && !allowMultiple) // getting checked newly
            for (int i=0; i<others.length; i++)
              others[i].checked = false;
          this.checked = true;
        }
        else checked = !checked;
      }
    } 
    else if (!mousePressed) {
      press = false;
    }
  }

  void release()
  {
    checked = false;
  }
  
  void allowMultiple(boolean allow)
  {
    allowMultiple = allow;
    if(others != null)
      for(int i=0; i<others.length; i++)
        others[i].allowMultiple = allow;
  }

  void display() 
  {

    over();
    press();
    fill(255);
    stroke(0.5);
    strokeWeight(1);
    rectMode(CORNER);
    rect(x, y, size, size);
    if (over || press || checked) {
      line(x, y, x+size, y+size);
      line(x, y+size, x+size, y);
    }
  }

  public boolean isChecked()
  {
    return checked;
  }
}

public class Slider {
  // original source: http://www.openprocessing.org/visuals/?visualID=29801

  PApplet parent;
  float x, y, redX, bigX, bigY;
  boolean mouseOver = false;

  boolean directionIsX = true;
  boolean showSegments = true;
  boolean oneEnded = false;
  color sliderColor = color(#e9b200);//978e43);
  color backgroundColor = color(#1F1F1F);
  color strokeColor = color(255);
  int lowestValue = 0;  // for now day time
  int highestValue = 24; // for now day time
  int segments = 24; // for now day time
  float sliderXY1 = 0; 
  float sliderXY2 = 0;

  public Slider(PApplet p, float x1, float y1, float x2, float y2) {
    parent = p;
    this.x = x1;
    this.y = y1;
    bigX = x2;
    bigY = y2;
    if (directionIsX)
    {
      sliderXY1 = x1;
      sliderXY2 = x2;
    }
    else
    {
      sliderXY1 = y1;
      sliderXY2 = y2;
    }
  }

  public void display() {
    rectMode(CORNERS);
    parent.noStroke();
    parent.smooth();
    parent.fill(backgroundColor);
    parent.rect(x, y, bigX, bigY);

    if (!oneEnded) // otherwise pivot one side with the respective vertex of (x,y) or (bigX,bigY)
    {
      if (directionIsX) // x is variable
      {
        //        float X1 = x + (bigX-x) * (sliderValueLow - lowestValue) / segments;
        //        float X2 = x + (bigX-x) * (highestValue - sliderValueHigh) / segments;
        parent.fill(sliderColor);
        parent.rect(sliderXY1, y, sliderXY2, bigY);
      }
    }
    // draw border again
    parent.stroke(strokeColor);
    parent.strokeWeight(1);
    parent.noFill(); 
    parent.rect(x, y, bigX, bigY);
    // draw segment markers
    if (showSegments)
    {
      parent.stroke(strokeColor);
      parent.strokeWeight(1);
      for (int i=0; i<=segments; i++)
      {
        if (directionIsX)
        {
          float X = x + (bigX-x) * i / segments;
          beginShape();
          vertex(X, y);
          vertex(X, bigY);
          endShape();
          fill(#FFFFFF);
          if (i%3 == 0)
          {
            textSize(13);
            textLeading(10);
            textAlign(CENTER);
            text(i, X, bigY+15); //
          }
        }
      }
    }
  }

  boolean locked1 = false;
  boolean locked2 = false;
  public void update() {
    if (parent.mouseX > x-1 && parent.mouseX < bigX && parent.mouseY > y && parent.mouseY < bigY)
      mouseOver = true;
    else 
      mouseOver = false;
    if (mouseOver && parent.mousePressed && !locked1 && !locked2)
    {
      if (parent.mouseX > sliderXY1-1 && parent.mouseX < sliderXY1 + 5) // 5 pixel tollerance
        locked1 = true;
      else if (parent.mouseX > sliderXY2 - 5 && parent.mouseX < sliderXY2 + 1) // 5 pixel tollerance
        locked2 = true;
    }
    if (locked1 && !oneEnded && parent.mouseX < sliderXY2 - 5) // tollerace 5
    {
      sliderXY1 = parent.mouseX;
      if (sliderXY1 < x) sliderXY1 = x;
    }
    else if (locked2 && parent.mouseX > sliderXY1 + 5) // tollerace 5
    {
      sliderXY2 = parent.mouseX;
      if (sliderXY2 > bigX) sliderXY2 = bigX;
    }

    if (!parent.mousePressed)
    {
      locked1 = locked2 = false;
    }
  }

  //  public float getValue() {
  //    return redX;
  //  }

  public float getLowerValue() {
    if (directionIsX) return map(sliderXY1, x, bigX, lowestValue, highestValue);
    else return map(sliderXY1, y, bigY, lowestValue, highestValue);
  }
  public float getHigherValue() {
    if (directionIsX) return map(sliderXY2, x, bigX, lowestValue, highestValue);
    else return map(sliderXY2, y, bigY, lowestValue, highestValue);
  }
}

public class OptionLabel
{
  int x, y;
  int width;
  int height;
  int textSize = 14;
  int textLeading = 15;
  color textColor = color(255);
  String text;
  boolean selected = false;
  PFont font, fontBold;

  boolean over;
  boolean press;
  boolean locked = false;
  boolean otherslocked = false;
  boolean allowMultiple = false;
  OptionLabel[] others;
  PApplet parent;

  OptionLabel(int x, int y, int width, int height, String text, OptionLabel[] others)
  {
    this.x = x;
    this.y = y;
    this.width = width;
    this.height = height;
    //    this.textSize = textSize;
    //    this.textLeading = textLeading;
    this.text = text;
    this.others = others;
    font = loadFont("Helvetica-48.vlw");
    fontBold = loadFont("Helvetica-Bold-48.vlw");
    //fontBold = createFont("Serif", 20);
    parent = null;
  }
  
    OptionLabel(int x, int y, int width, int height, String text, OptionLabel[] others, PApplet p)
  {
    this.x = x;
    this.y = y;
    this.width = width;
    this.height = height;
    //    this.textSize = textSize;
    //    this.textLeading = textLeading;
    this.text = text;
    this.others = others;
    font = loadFont("Helvetica-48.vlw");
    fontBold = loadFont("Helvetica-Bold-48.vlw");
    //fontBold = createFont("Serif", 20);
    parent = p;
  }

  void update()
  {
    for (int i=0; i<others.length; i++)
    {
//      if (others[i].locked)
//      {
//        otherslocked = true;
//        break;
//      }
//      else
//      {
//        otherslocked = false;
//      }
    }

    if (!otherslocked)
    {
      over();
      press();
    }

    if (press) {
      // make others unselected
      if(!allowMultiple)
      for (int i=0; i<others.length; i++)
      {
        others[i].selected = false;
      }
      this.selected = !allowMultiple || !selected;
      // temporary work around for the case deselecting the "All Year" in parent
//      if(parent != null && this != parent.allYear)
//        parent.allYear.selected = false;
    }
  }

  void over()
  {
    if (overRect(x, y, width, height)) {
      over = true;
    } 
    else {
      over = false;
    }
  }

  void press()
  {
    if (over && mousePressed && !locked) {
      press = true;
      locked = true;
    } 
    else {
      press = false;
    }
  }

  void release()
  {
    locked = false;
  }
  
  void allowMultiple(boolean allow)
  {
    allowMultiple = allow;
    if(others != null)
      for(int i=0; i<others.length; i++)
        others[i].allowMultiple = allow;
  }

  void display() 
  {
    if (over || press || selected) {
      textFont(fontBold);
      fill(textColor);
    }
    else {
      textFont(font);
      fill(200);
    
    }
    textSize(this.textSize);
    textLeading(this.textLeading);
    textAlign(CENTER, CENTER);
    text(this.text, x+width/2, y+height/2);
    //println("-" + this.text);
    if(!mousePressed) release();
  }
}

